#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Plots the dynamic support/resistance
    /// </summary>
    [Description("Plots the dynamic support/resistance")]
    [Gui.Design.DisplayName("Dynamic S/R")]
    public class DynamicSR : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int			period				= 21; 
		private DataSeries HMASeries;
        
		// User defined variables (add any user defined variables below)

		private double		DynamicR			= 0;
		private double		DynamicS			= 0;
		private double		HH					=0;
		private double		LL					=0;
		private int			barcounter			= 0;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.Blue,3), PlotStyle.Line, "Resistance"));
			Add(new Plot(new Pen(Color.Magenta,3), PlotStyle.Line, "Support"));
									
			HMASeries			= new DataSeries(this);
			
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
			PaintPriceMarkers	= true;
			AutoScale			= false;
			BarsRequired		= 1;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            HMASeries.Set(HMA(period)[0]);
						
			if ( CurrentBar < 2 )	return;
			{	
				HH = High[Bars.HighestBar(10)];
				if (HMASeries[1] > HMASeries[2] && HMASeries[0] < HMASeries[1])
				 	DynamicR=HH;
				LL = Low[Bars.LowestBar(10)];
				if(HMASeries[1]< HMASeries[2] && HMASeries[0] > HMASeries[1])
					DynamicS=LL;
			}
						
			Support.Set(DynamicS);
			Resistance.Set(DynamicR);
	
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Resistance
        {
            get { return Values[0]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Support
        {
            get { return Values[1]; }
        }

		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private DynamicSR[] cacheDynamicSR = null;

        private static DynamicSR checkDynamicSR = new DynamicSR();

        /// <summary>
        /// Plots the dynamic support/resistance
        /// </summary>
        /// <returns></returns>
        public DynamicSR DynamicSR(int period)
        {
            return DynamicSR(Input, period);
        }

        /// <summary>
        /// Plots the dynamic support/resistance
        /// </summary>
        /// <returns></returns>
        public DynamicSR DynamicSR(Data.IDataSeries input, int period)
        {
            checkDynamicSR.Period = period;
            period = checkDynamicSR.Period;

            if (cacheDynamicSR != null)
                for (int idx = 0; idx < cacheDynamicSR.Length; idx++)
                    if (cacheDynamicSR[idx].Period == period && cacheDynamicSR[idx].EqualsInput(input))
                        return cacheDynamicSR[idx];

            DynamicSR indicator = new DynamicSR();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;
            indicator.SetUp();

            DynamicSR[] tmp = new DynamicSR[cacheDynamicSR == null ? 1 : cacheDynamicSR.Length + 1];
            if (cacheDynamicSR != null)
                cacheDynamicSR.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheDynamicSR = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Plots the dynamic support/resistance
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DynamicSR DynamicSR(int period)
        {
            return _indicator.DynamicSR(Input, period);
        }

        /// <summary>
        /// Plots the dynamic support/resistance
        /// </summary>
        /// <returns></returns>
        public Indicator.DynamicSR DynamicSR(Data.IDataSeries input, int period)
        {
            return _indicator.DynamicSR(input, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Plots the dynamic support/resistance
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DynamicSR DynamicSR(int period)
        {
            return _indicator.DynamicSR(Input, period);
        }

        /// <summary>
        /// Plots the dynamic support/resistance
        /// </summary>
        /// <returns></returns>
        public Indicator.DynamicSR DynamicSR(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.DynamicSR(input, period);
        }

    }
}
#endregion
