// NinjaTrader port of 1998 version of Jurik Moving Average
#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Jurik Moving Average
    /// </summary>
    [Description("Jurik Moving Average")]
    public class JMA : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int length = 14; // Default setting for Length
            private double phase = 0; // Default setting for Phase
        // User defined variables (add any user defined variables below)
		DataSeries JMAValueBuffer, fC0Buffer, fA8Buffer, fC8Buffer;
		//---- temporary buffers
		double[] list, ring1, ring2, buffer;
		int    limitValue, startValue, loopParam, loopCriteria;
		int    cycleLimit, highLimit, counterA, counterB;
		bool initFlag;
		double cycleDelta, lowDValue, highDValue, absValue, paramA, paramB;
		double phaseParam, logParam, JMAValue, series, sValue, sqrtParam, lengthDivider, dValue=0.0;
//---- temporary int variables
		int   s58, s60, s40, s38, s68;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot (new Pen(Color.FromKnownColor(KnownColor.Orchid),2), PlotStyle.Line, "JMA"));

			int i;
			list = new double[128]; // initializes to zero
			ring1 = new double[128];
			ring2 = new double[11];
			buffer = new double[62];

			JMAValueBuffer = new DataSeries(this);
			fC0Buffer = new DataSeries(this);
			fA8Buffer = new DataSeries(this);
			fC8Buffer = new DataSeries(this);

			limitValue = 63; 
			startValue = 64;
			loopParam = loopCriteria = 0;

			for (i = 0; i <= limitValue; i++) list [i] = -1000000; 
			for (i = startValue; i <= 127; i++) list [i] = 1000000; 

			initFlag  = true;
			double lengthParam = (Length < 1.0000000002) ? 0.0000000001 : (Length - 1) / 2.0;

			if (Phase < -100) phaseParam = 0.5;
			else if (Phase > 100) phaseParam = 2.5;
			else phaseParam = Phase / 100.0 + 1.5;

			logParam = Math.Log(Math.Sqrt(lengthParam)) / Math.Log(2.0);
			if (logParam + 2.0 < 0) logParam = 0;
			else logParam += 2.0; 

			sqrtParam = Math.Sqrt(lengthParam) * logParam; 
			lengthParam *= 0.9; 
			lengthDivider = lengthParam / (lengthParam + 2.0);
			
            Overlay				= true;
            PriceTypeSupported	= true;	
			CalculateOnBarClose	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			int i, j, limit = CurrentBar;
			double JMATempValue = (CurrentBar > 0) ? JMAValueBuffer[1] : Input[0];

			series = Input[0];
			if (loopParam < 61) { 
				loopParam++; 
				buffer[loopParam] = series;
			}
			if (loopParam > 30) {
				if (initFlag) {
					initFlag = false;
					int diffFlag = 0;
					for (i = 1; i <= 29; i++) if (buffer[i+1] != buffer[i]) diffFlag = 1;
					highLimit = diffFlag * 30;
					paramB = (highLimit == 0) ? series : buffer[1];
					paramA = paramB;
					if (highLimit > 29) highLimit = 29;
				} else highLimit = 0;
//---- big cycle
			for (i = highLimit; i >= 0; i--) { 
				sValue = (i==0) ? series : buffer[31-i];
				absValue = (Math.Abs(sValue - paramA) > Math.Abs(sValue - paramB)) ?
					Math.Abs(sValue - paramA) : Math.Abs(sValue - paramB); 
				dValue = absValue + 0.0000000001; //1.0e-10;
				if (counterA <= 1) counterA = 127; else counterA--; //starts at 127
				if (counterB <= 1) counterB = 10;  else counterB--; //starts at 10
				if (cycleLimit < 128) cycleLimit++; 
				cycleDelta += (dValue - ring2 [counterB]); 
				ring2 [counterB] = dValue; 
				highDValue = (cycleLimit > 10) ? cycleDelta / 10.0 : cycleDelta / cycleLimit; 

				if (cycleLimit > 127) { 
					dValue = ring1 [counterA]; 
					ring1 [counterA] = highDValue; 
					s68 = 64; s58 = s68; 
					while (s68 > 1) { 
						if (list [s58] < dValue)
							{ s68 /= 2; s58 += s68; }
						else if (list [s58] <= dValue)
						   s68 = 1; 
						else { s68 /= 2; s58 -= s68; }
					}
				} else {
					ring1[counterA] = highDValue; 
					if ((limitValue + startValue) > 127) {
						startValue--; 
						s58 = startValue; 
					} else {
						limitValue++; 
						s58 = limitValue; 
					}
					s38 = (limitValue > 96) ? 96 : limitValue; 
					s40 = (startValue < 32) ? 32 : startValue; 
		      }
//----
				s68 = 64; s60 = s68; 
				while (s68 > 1) {
					if (list[s60] >= highDValue) {
						if (list[s60 - 1] <= highDValue) s68 = 1; 
						else {
							s68 /= 2; 
							s60 -= s68; 
						}
					} else {
						s68 /= 2; 
						s60 += s68; 
					}
					if ((s60 == 127) && (highDValue > list[127])) s60 = 128; 
				}
				if (cycleLimit > 127) {
					if (s58 >= s60) {
						if (((s38 + 1) > s60) && ((s40 - 1) < s60)) 
							lowDValue += highDValue; 
						else if ((s40 > s60) && ((s40 - 1) < s58)) 
							lowDValue += list [s40 - 1]; 
					}
					else if (s40 >= s60) {
						if (((s38 + 1) < s60) && ((s38 + 1) > s58)) 
							lowDValue += list[s38 + 1]; 
					}
					else if ((s38 + 2) > s60) 
						lowDValue += highDValue; 
					else if (((s38 + 1) < s60) && ((s38 + 1) > s58)) 
						lowDValue += list[s38 + 1]; 

					if (s58 > s60) {
						if (((s40 - 1) < s58) && ((s38 + 1) > s58)) 
							lowDValue -= list [s58]; 
						else if ((s38 < s58) && ((s38 + 1) > s60)) 
							lowDValue -= list [s38]; 
					} else {
						if (((s38 + 1) > s58) && ((s40 - 1) < s58)) 
							lowDValue -= list [s58]; 
						else if ((s40 > s58) && (s40 < s60)) 
							lowDValue -= list [s40]; 
					}
				}
				if (s58 <= s60) {
					if (s58 >= s60) list[s60] = highDValue; else {
						for (j = s58 + 1; j <= (s60 - 1); j++) list [j-1] = list[j]; 
						list [s60 - 1] = highDValue; 
					}
				} else {
					for (j = s58 - 1; j >= s60; j--) list [j + 1] = list [j]; 
					list [s60] = highDValue; 
				}

				if (cycleLimit <= 127) {
					lowDValue = 0; 
					for (j = s40; j <= s38; j++) lowDValue += list[j]; 
				}
//----
				if (CurrentBar == 0) {
					fC0Buffer.Set(series);
					JMATempValue = series;
					int leftInt = (Math.Ceiling(sqrtParam) >= 1) ? (int)Math.Ceiling(sqrtParam) : 1; 
					int rightPart = (Math.Floor(sqrtParam) >= 1) ? (int)Math.Floor(sqrtParam) : 1; 
					dValue = (leftInt == rightPart) ? 1.0
						: (sqrtParam - rightPart) / (leftInt - rightPart);
				}

				if ((loopCriteria + 1) > 31) loopCriteria = 31; else loopCriteria++; 
				double sqrtDivider = sqrtParam / (sqrtParam + 1.0);

				if (loopCriteria <= 30) {
					paramA = (sValue - paramA > 0) ? sValue : sValue - (sValue - paramA) * sqrtDivider; 
					paramB = (sValue - paramB < 0) ? sValue : sValue - (sValue - paramB) * sqrtDivider; 
					JMATempValue = series;

					if (loopCriteria == 30) { 
						fC0Buffer[0] = series;
						int intPart = (Math.Ceiling(sqrtParam) >= 1.0) ? (int)Math.Ceiling(sqrtParam) : 1; 
						int dnShift, leftInt = intPart; //IntPortion(intPart); 
						intPart = (Math.Floor(sqrtParam) >= 1.0) ? (int)Math.Floor(sqrtParam) : 1; 
						int upShift, rightPart = intPart; //IntPortion (intPart);
						dValue = (leftInt == rightPart) ? 1.0
							: (sqrtParam - rightPart) / (leftInt - rightPart);
						upShift = (rightPart <= 29) ? rightPart : 29; 
						dnShift = (leftInt <= 29) ? leftInt : 29; 
						fA8Buffer[0] = (series - buffer [loopParam - upShift]) * (1 - dValue) / rightPart + (series - buffer[loopParam - dnShift]) * dValue / leftInt;
					}
				} else {
					double powerValue, squareValue;
					dValue = lowDValue / (s38 - s40 + 1);
					powerValue = (0.5 <= logParam - 2.0) ? logParam - 2.0 : 0.5;
					if (logParam >= Math.Pow(absValue/dValue, powerValue))
						dValue = Math.Pow(absValue/dValue, powerValue);
					else dValue = logParam; 
					if (dValue < 1) dValue = 1;
					powerValue = Math.Pow(sqrtDivider, Math.Sqrt(dValue)); 
					paramA = (sValue - paramA > 0) ? sValue : sValue - (sValue - paramA) * powerValue;
					paramB = (sValue - paramB < 0) ? sValue : sValue - (sValue - paramB) * powerValue; 
   				}
			}
// ---- end of big cycle                  			   
			if (loopCriteria > 30) {
				double powerValue   = Math.Pow(lengthDivider, dValue);
				double squareValue  = powerValue * powerValue;
				fC0Buffer[0] = (1 - powerValue) * series + powerValue * fC0Buffer[1];
				fC8Buffer[0] = (series - fC0Buffer[0]) * (1.0 - lengthDivider) + lengthDivider * fC8Buffer[1];
				fA8Buffer[0] = (phaseParam * fC8Buffer[0] + fC0Buffer[0] - JMATempValue) *
					(powerValue * (-2.0) + squareValue + 1) + squareValue * fA8Buffer[1];  
				JMATempValue += fA8Buffer[0]; 
			}
			JMAValue = JMATempValue;
		} //endif loopparam>30
		if (loopParam <= 30) JMAValue = Input[0];
		else Value.Set(JMAValue);
		JMAValueBuffer[0] = JMAValue;
        }

        #region Properties

        [Description("Lookback interval")]
        [Category("Parameters")]
        public int Length
        {
            get { return length; }
            set { length = Math.Max(1, value); }
        }

        [Description("Phase (similar to offset)")]
        [Category("Parameters")]
        public double Phase
        {
            get { return phase; }
            set { phase = Math.Max(-100, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private JMA[] cacheJMA = null;

        private static JMA checkJMA = new JMA();

        /// <summary>
        /// Jurik Moving Average
        /// </summary>
        /// <returns></returns>
        public JMA JMA(int length, double phase)
        {
            return JMA(Input, length, phase);
        }

        /// <summary>
        /// Jurik Moving Average
        /// </summary>
        /// <returns></returns>
        public JMA JMA(Data.IDataSeries input, int length, double phase)
        {
            checkJMA.Length = length;
            length = checkJMA.Length;
            checkJMA.Phase = phase;
            phase = checkJMA.Phase;

            if (cacheJMA != null)
                for (int idx = 0; idx < cacheJMA.Length; idx++)
                    if (cacheJMA[idx].Length == length && Math.Abs(cacheJMA[idx].Phase - phase) <= double.Epsilon && cacheJMA[idx].EqualsInput(input))
                        return cacheJMA[idx];

            JMA indicator = new JMA();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Length = length;
            indicator.Phase = phase;
            indicator.SetUp();

            JMA[] tmp = new JMA[cacheJMA == null ? 1 : cacheJMA.Length + 1];
            if (cacheJMA != null)
                cacheJMA.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheJMA = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Jurik Moving Average
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.JMA JMA(int length, double phase)
        {
            return _indicator.JMA(Input, length, phase);
        }

        /// <summary>
        /// Jurik Moving Average
        /// </summary>
        /// <returns></returns>
        public Indicator.JMA JMA(Data.IDataSeries input, int length, double phase)
        {
            return _indicator.JMA(input, length, phase);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Jurik Moving Average
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.JMA JMA(int length, double phase)
        {
            return _indicator.JMA(Input, length, phase);
        }

        /// <summary>
        /// Jurik Moving Average
        /// </summary>
        /// <returns></returns>
        public Indicator.JMA JMA(Data.IDataSeries input, int length, double phase)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.JMA(input, length, phase);
        }

    }
}
#endregion
